/***************************************************
 * Edison Work Shop "Cloud" Soil
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=76  LM35 Analog Linear Temperature Sensor
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=599  Soil Moisture Sensor
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=1198 Intel® Edison with Arduino Breakout Kit
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=135  IIC LCD1602
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=1076  Magnet Micro USB Cable 1.2m
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=1009  IO Expansion Shield for Arduino V7.1
 * >http://www.dfrobot.com/index.php?route=product/product&product_id=666  IIC LCD module dedicated cable
 *
 ***************************************************
 * This example reads temperature and humidity, shows the data on 1602 IIC LCD and send them to the wilink >http://www.wilink.cc
 *
 * Created 2014-12-8
 * By Angelo qiao <Angelo.qiao@dfrobot.com>
 * Modified 2014-12-8
 * By Angelo qiao Angelo.qiao@dfrobot.com>
 *
 * GNU Lesser General Public License.
 * See <http://www.gnu.org/licenses/> for details.
 * All above must be included in any redistribution
 ****************************************************/

/***********Notice and Trouble shooting***************
 * 1.This code is tested on Intel® Edison.
 * 2.LiquidCrystal_I2C library is created by DFRobot.
 * >http://www.dfrobot.com/image/data/DFR0154/LiquidCrystal_I2Cv1-1.rar
 ****************************************************/

#include <SPI.h>
#include <WiFi.h>
#include <LiquidCrystal.h>

String yourName = "AngeloQiao";

char ssid[] = "Inno Space2";      //  your network SSID (name)
char pass[] = "cube3cube3";   // your network password
int keyIndex = 0;            // your network key Index number (needed only for WEP)

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

int status = WL_IDLE_STATUS;

#define WEBSITE  "api.xively.com"
#define API_key  "mr2elMOsg9gHJhlgHn2IoKqSX52TmcYUEVmaPvsJviE70GG6"  // Update Your API Key
#define feedID  "96606792"                                         // Update Your own feedID


int TemperaturePin=A1;      //设置LM35线性温度传感器的引脚
int HumidityPin=A2;         //Set the pin number of the Soil Moisture Sensor

void printWifiStatus();

// Initialize the Wifi client library
WiFiClient client;

// server address:
IPAddress server(216,52,233,120);

unsigned long lastConnectionTime = 0;           // last time you connected to the server, in milliseconds
boolean lastConnected = false;                  // state of the connection last time through the main loop
const unsigned long postingInterval = 5000;  // delay between updates, in milliseconds

int TEMPERATURE=0,HUMIDITY=0;


void setup() {
    //Initialize serial and wait for port to open:
    Serial.begin(9600);
    //  while (!Serial) {
    //    ; // wait for serial port to connect. Needed for Leonardo only
    //  }
    
    for(int i = 4; i<=9; i++)
    {
        pinMode(i, OUTPUT);
    }
    
    
    lcd.begin(16,2); //设置LCD为2行,每行16个字符
    lcd.setCursor(0, 0);             //光标移到第一行,第一个字符
    lcd.print("Wifi Connecting.");
    
    
    // check for the presence of the shield:
    if (WiFi.status() == WL_NO_SHIELD) {
        Serial.println("WiFi shield not present");
        // don't continue:
        while(true);
    }
    
    String fv = WiFi.firmwareVersion();
    if( fv != "1.1.0" )
        Serial.println("Please upgrade the firmware");
    
    // attempt to connect to Wifi network:
    while ( status != WL_CONNECTED) {
        Serial.print("Attempting to connect to SSID: ");
        Serial.println(ssid);
        // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
        status = WiFi.begin(ssid, pass);
        
        // wait 10 seconds for connection:
        delay(10000);
    }
    // you're connected now, so print out the status:
    printWifiStatus();
    
    lcd.setCursor(0, 0);    //光标移到第一行,第一个字符
    lcd.print("                ");  //clear the first line
}


void loop() {
    
    // if there's no net connection, but there was one last time
    // through the loop, then stop the client:
    if (!client.connected() && lastConnected) {
        //    Serial.println();
        //    Serial.println("disconnecting.");
        client.stop();
    }
    
    
    static unsigned long sensorTimer = millis();
    if(millis()-sensorTimer>1000)
    {
        sensorTimer = millis();
        long temperatureValue=analogRead(TemperaturePin);    //读取温度的模拟量
        long humidityValue=analogRead(HumidityPin);          //读取湿度的模拟量
        
        TEMPERATURE=(500 * temperatureValue) /1024;
        HUMIDITY=humidityValue;
        
        lcd.setCursor(0, 0);
        lcd.print("T:");
        lcd.print(TEMPERATURE);
        lcd.print("C");
        lcd.setCursor(6, 0);
        lcd.print("H:");
        lcd.print(HUMIDITY);
        lcd.setCursor(0, 1);
        if (HUMIDITY < 300) {
            lcd.print("Soil: Dry  ");
        }
        else if (HUMIDITY >= 300 && HUMIDITY < 700) {
            lcd.print("Soil: Humid");
        }
        else {
            lcd.print("Soil: Water");
        }
    }
    
    
    
    // if you're not connected, and ten seconds have passed since
    // your last connection, then connect again and send data:
    if(!client.connected() && (millis() - lastConnectionTime > postingInterval)) {
        httpRequestTEMPERATURE();
        httpRequestHUMIDITY();
    }
    // store the state of the connection for next time through
    // the loop:
    lastConnected = client.connected();
}

// this method makes a HTTP connection to the server:
void httpRequestTEMPERATURE() {
    // if there's a successful connection:
    if (client.connect(server, 80)) {
        //    Serial.println("connecting...");
        
        String data_temperature,data_start;
        
        // Prepare JSON for Xively & get length
        int length = 0;
        // JSON beginning
        data_start = "";
        data_start = data_start + "\n"
        + "{\"version\":\"1.0.0\",\"datastreams\" : [ ";
        // JSON for temperature & humidity
        data_temperature = "{\"id\" : \""+yourName+"Temperature\",\"current_value\" : \""
        + String(int(TEMPERATURE)) + "\"}]}";
        // Get length
        length = data_start.length() + data_temperature.length();
        
        //    Serial.println("Connected to Xively server.");
        
        // Send headers
        //    Serial.print("Sending headers");
        client.print("PUT /v2/feeds/");
        client.print(feedID);
        client.println(".json HTTP/1.0");
        //Serial.print(".");
        client.println(("Host: api.xively.com"));
        //Serial.print(("."));
        client.print(("X-ApiKey: "));
        client.println(API_key);
        //Serial.print(("."));
        client.print(("Content-Length: "));
        client.println(length);
        //Serial.print(("."));
        client.print(("Connection: close"));
        //Serial.println((" done."));
        
        // Send data
        //Serial.print(("Sending data"));
        client.println((""));
        client.print(data_start);
        //Serial.print(("."));
        client.print(data_temperature);
        //Serial.print(("."));
        client.println((""));
        //Serial.println((" done."));
        
        
        
        
        //read the feedback from the server
        if (client.available() && client.connected() ) {
            while(true)
            {
                int c = client.read();
                if(c==-1)
                {
                    break;
                }
                //Serial.print((char)c);
            }
        }
        
        client.stop();
        lastConnectionTime = millis();
    }
    else {
        // if you couldn't make a connection:
        //Serial.println("connection failed");
        //Serial.println("disconnecting.");
        client.stop();
    }
}

void httpRequestHUMIDITY() {
    // if there's a successful connection:
    if (client.connect(server, 80)) {
        //Serial.println("connecting...");
        
        String data_temperature,data_start;
        
        
        // Prepare JSON for Xively & get length
        int length = 0;
        // JSON beginning
        data_start = "";
        data_start = data_start + "\n"
        + "{\"version\":\"1.0.0\",\"datastreams\" : [ ";
        // JSON for temperature & humidity
        data_temperature = "{\"id\" : \""+yourName+"Humidity\",\"current_value\" : \""
        + String(int(HUMIDITY)) + "\"}]}";
        // Get length
        length = data_start.length() + data_temperature.length();
        
        //Serial.println("Connected to Xively server.");
        
        // Send headers
        //Serial.print("Sending headers");
        client.print("PUT /v2/feeds/");
        client.print(feedID);
        client.println(".json HTTP/1.0");
        //Serial.print(".");
        client.println(("Host: api.xively.com"));
        //Serial.print(("."));
        client.print(("X-ApiKey: "));
        client.println(API_key);
        //Serial.print(("."));
        client.print(("Content-Length: "));
        client.println(length);
        //Serial.print(("."));
        client.print(("Connection: close"));
        //Serial.println((" done."));
        
        // Send data
        //Serial.print(("Sending data"));
        client.println((""));
        client.print(data_start);
        //Serial.print(("."));
        client.print(data_temperature);
        //Serial.print(("."));
        client.println((""));
        //Serial.println((" done."));
        
        //read the feedback from the server
        if (client.available() && client.connected() ) {
            while(true)
            {
                int c = client.read();
                if(c==-1)
                {
                    break;
                }
                //Serial.print((char)c);
            }
        }
        
        client.stop();
        lastConnectionTime = millis();
    }
    else {
        // if you couldn't make a connection:
        //Serial.println("connection failed");
        //Serial.println("disconnecting.");
        client.stop();
    }
}

void printWifiStatus() {
    // print the SSID of the network you're attached to:
    Serial.print("SSID: ");
    Serial.println(WiFi.SSID());
    
    // print your WiFi shield's IP address:
    IPAddress ip = WiFi.localIP();
    Serial.print("IP Address: ");
    Serial.println(ip);
    
    // print the received signal strength:
    long rssi = WiFi.RSSI();
    Serial.print("signal strength (RSSI):");
    Serial.print(rssi);
    Serial.println(" dBm");
}

